@@ -31,6 +31,7 @@
 #include "src/core/ext/filters/client_channel/client_channel.h"
 #include "src/core/ext/filters/client_channel/lb_policy_registry.h"
 #include "src/core/ext/filters/client_channel/server_address.h"
+#include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/status_util.h"
 #include "src/core/lib/gpr/string.h"
 #include "src/core/lib/gprpp/memory.h"
@@ -43,42 +44,64 @@ namespace grpc_core {
 namespace internal {
 
 ProcessedResolverResult::ProcessedResolverResult(
-    const grpc_channel_args& resolver_result, bool parse_retry) {
-  ProcessServiceConfig(resolver_result, parse_retry);
+    Resolver::Result* resolver_result, bool parse_retry)
+    : service_config_(resolver_result->service_config) {
+  // If resolver did not return a service config, use the default
+  // specified via the client API.
+  if (service_config_ == nullptr) {
+    const char* service_config_json = grpc_channel_arg_get_string(
+        grpc_channel_args_find(resolver_result->args, GRPC_ARG_SERVICE_CONFIG));
+    if (service_config_json != nullptr) {
+      service_config_ = ServiceConfig::Create(service_config_json);
+    }
+  } else {
+    // Add the service config JSON to channel args so that it's
+    // accessible in the subchannel.
+    // TODO(roth): Consider whether there's a better way to pass the
+    // service config down into the subchannel stack, such as maybe via
+    // call context or metadata.  This would avoid the problem of having
+    // to recreate all subchannels whenever the service config changes.
+    // It would also avoid the need to pass in the resolver result in
+    // mutable form, both here and in
+    // ResolvingLoadBalancingPolicy::ProcessResolverResultCallback().
+    grpc_arg arg = grpc_channel_arg_string_create(
+        const_cast<char*>(GRPC_ARG_SERVICE_CONFIG),
+        const_cast<char*>(service_config_->service_config_json()));
+    grpc_channel_args* new_args =
+        grpc_channel_args_copy_and_add(resolver_result->args, &arg, 1);
+    grpc_channel_args_destroy(resolver_result->args);
+    resolver_result->args = new_args;
+  }
+  // Process service config.
+  ProcessServiceConfig(*resolver_result, parse_retry);
   // If no LB config was found above, just find the LB policy name then.
-  if (lb_policy_name_ == nullptr) ProcessLbPolicyName(resolver_result);
+  if (lb_policy_name_ == nullptr) ProcessLbPolicyName(*resolver_result);
 }
 
 void ProcessedResolverResult::ProcessServiceConfig(
-    const grpc_channel_args& resolver_result, bool parse_retry) {
-  const grpc_arg* channel_arg =
-      grpc_channel_args_find(&resolver_result, GRPC_ARG_SERVICE_CONFIG);
-  const char* service_config_json = grpc_channel_arg_get_string(channel_arg);
-  if (service_config_json != nullptr) {
-    service_config_json_.reset(gpr_strdup(service_config_json));
-    service_config_ = grpc_core::ServiceConfig::Create(service_config_json);
-    if (service_config_ != nullptr) {
-      if (parse_retry) {
-        channel_arg =
-            grpc_channel_args_find(&resolver_result, GRPC_ARG_SERVER_URI);
-        const char* server_uri = grpc_channel_arg_get_string(channel_arg);
-        GPR_ASSERT(server_uri != nullptr);
-        grpc_uri* uri = grpc_uri_parse(server_uri, true);
-        GPR_ASSERT(uri->path[0] != '\0');
-        server_name_ = uri->path[0] == '/' ? uri->path + 1 : uri->path;
-        service_config_->ParseGlobalParams(ParseServiceConfig, this);
-        grpc_uri_destroy(uri);
-      } else {
-        service_config_->ParseGlobalParams(ParseServiceConfig, this);
-      }
-      method_params_table_ = service_config_->CreateMethodConfigTable(
-          ClientChannelMethodParams::CreateFromJson);
-    }
+    const Resolver::Result& resolver_result, bool parse_retry) {
+  if (service_config_ == nullptr) return;
+  service_config_json_ =
+      UniquePtr<char>(gpr_strdup(service_config_->service_config_json()));
+  if (parse_retry) {
+    const grpc_arg* channel_arg =
+        grpc_channel_args_find(resolver_result.args, GRPC_ARG_SERVER_URI);
+    const char* server_uri = grpc_channel_arg_get_string(channel_arg);
+    GPR_ASSERT(server_uri != nullptr);
+    grpc_uri* uri = grpc_uri_parse(server_uri, true);
+    GPR_ASSERT(uri->path[0] != '\0');
+    server_name_ = uri->path[0] == '/' ? uri->path + 1 : uri->path;
+    service_config_->ParseGlobalParams(ParseServiceConfig, this);
+    grpc_uri_destroy(uri);
+  } else {
+    service_config_->ParseGlobalParams(ParseServiceConfig, this);
   }
+  method_params_table_ = service_config_->CreateMethodConfigTable(
+      ClientChannelMethodParams::CreateFromJson);
 }
 
 void ProcessedResolverResult::ProcessLbPolicyName(
-    const grpc_channel_args& resolver_result) {
+    const Resolver::Result& resolver_result) {
   // Prefer the LB policy name found in the service config. Note that this is
   // checking the deprecated loadBalancingPolicy field, rather than the new
   // loadBalancingConfig field.
@@ -96,32 +119,28 @@ void ProcessedResolverResult::ProcessLbPolicyName(
   // Otherwise, find the LB policy name set by the client API.
   if (lb_policy_name_ == nullptr) {
     const grpc_arg* channel_arg =
-        grpc_channel_args_find(&resolver_result, GRPC_ARG_LB_POLICY_NAME);
+        grpc_channel_args_find(resolver_result.args, GRPC_ARG_LB_POLICY_NAME);
     lb_policy_name_.reset(gpr_strdup(grpc_channel_arg_get_string(channel_arg)));
   }
   // Special case: If at least one balancer address is present, we use
   // the grpclb policy, regardless of what the resolver has returned.
-  const ServerAddressList* addresses =
-      FindServerAddressListChannelArg(&resolver_result);
-  if (addresses != nullptr) {
-    bool found_balancer_address = false;
-    for (size_t i = 0; i < addresses->size(); ++i) {
-      const ServerAddress& address = (*addresses)[i];
-      if (address.IsBalancer()) {
-        found_balancer_address = true;
-        break;
-      }
+  bool found_balancer_address = false;
+  for (size_t i = 0; i < resolver_result.addresses.size(); ++i) {
+    const ServerAddress& address = resolver_result.addresses[i];
+    if (address.IsBalancer()) {
+      found_balancer_address = true;
+      break;
     }
-    if (found_balancer_address) {
-      if (lb_policy_name_ != nullptr &&
-          strcmp(lb_policy_name_.get(), "grpclb") != 0) {
-        gpr_log(GPR_INFO,
-                "resolver requested LB policy %s but provided at least one "
-                "balancer address -- forcing use of grpclb LB policy",
-                lb_policy_name_.get());
-      }
-      lb_policy_name_.reset(gpr_strdup("grpclb"));
+  }
+  if (found_balancer_address) {
+    if (lb_policy_name_ != nullptr &&
+        strcmp(lb_policy_name_.get(), "grpclb") != 0) {
+      gpr_log(GPR_INFO,
+              "resolver requested LB policy %s but provided at least one "
+              "balancer address -- forcing use of grpclb LB policy",
+              lb_policy_name_.get());
     }
+    lb_policy_name_.reset(gpr_strdup("grpclb"));
   }
   // Use pick_first if nothing was specified and we didn't select grpclb
   // above.
@@ -148,7 +167,8 @@ void ProcessedResolverResult::ParseLbConfigFromServiceConfig(
       LoadBalancingPolicy::ParseLoadBalancingConfig(field);
   if (policy != nullptr) {
     lb_policy_name_.reset(gpr_strdup(policy->key));
-    lb_policy_config_ = policy->child;
+    lb_policy_config_ =
+        MakeRefCounted<LoadBalancingPolicy::Config>(policy, service_config_);
   }
 }
 
