@@ -77,10 +77,18 @@ static bool iomgr_platform_is_any_background_poller_thread(void) {
   return false;
 }
 
+static bool iomgr_platform_add_closure_to_background_poller(
+    grpc_closure* closure, grpc_error* error) {
+  return false;
+}
+
 static grpc_iomgr_platform_vtable vtable = {
-    iomgr_platform_init, iomgr_platform_flush, iomgr_platform_shutdown,
+    iomgr_platform_init,
+    iomgr_platform_flush,
+    iomgr_platform_shutdown,
     iomgr_platform_shutdown_background_closure,
-    iomgr_platform_is_any_background_poller_thread};
+    iomgr_platform_is_any_background_poller_thread,
+    iomgr_platform_add_closure_to_background_poller};
 
 void grpc_set_default_iomgr_platform() {
   grpc_set_tcp_client_impl(&grpc_windows_tcp_client_vtable);
